/**
 * This thread is for testing and just does a local lookup.
 * @author Christian Grothoff
 * @file gproxy/LoopBuildThread.java
 **/
package gproxy;

import java.net.*;
import java.io.*;

/**
 * This thread is for testing and just does a local lookup.
 * @author Christian Grothoff
 **/
public class LoopbackBuildThread 
    extends BuildThread {

    private final String directory_;
 
    /**
     * Read files from the disk (for testing)
     * @param dir the directory to read from (SOMETHING/data/content/)
     **/
    public LoopbackBuildThread(String dir) {
	super('\0',null);
	this.directory_ = dir;
    }

    /**
     * Don't do anything!
     **/
    public void run(){
    }
    
    /**
     * Add a listener.
     * @param l the listener 
     **/
    public void addListener(Listener l) {
	synchronized (listeners_) {
	    if (listeners_.length == listCount_) {
		Listener[] tmp = new Listener[listCount_*2];
		for (int i=0;i<listCount_;i++)
                    tmp[i] = listeners_[i];
		listeners_ = tmp;
	    }
	    listeners_[listCount_++] = l;
	}           
    }
    
    /**
     * Remove a listener.
     * @param l the listener 
     **/
    public void removeListener(Listener l) {
	synchronized (listeners_) {
	    for (int i=0;i<listCount_;i++)
                 if (listeners_[i] == l) {
                     listeners_[i] = listeners_[--listCount_];
		     return;
                 }
	}
    }  

    public static String bytesToHex(byte[] msg) {
	String hexTable = "0123456789ABCDEF";
 	StringBuffer st = new StringBuffer(16);
	for (int i=0;i<msg.length;i++) {
	    st.append(hexTable.charAt(((char)msg[i]) & 15));
	    st.append(hexTable.charAt((((char)msg[i])>>4) & 15));
	}
	return st.toString();
    }
	
    /**
     * Send a request to GNet. 
     * @param message the request that should be send (hashcode / filename)
     * @param priority the priority of the request...
     * @param ttl the time-to-live for the request
     **/
    public void sendData (byte[] message,
			  int priority,
			  int ttl) {
	String st = bytesToHex(message);
	try {
	    FileInputStream fis = new FileInputStream(directory_+st.toString());
	    // System.out.println("Looking for file " + st.toString());
	    byte[] buffer 
		= new byte[Constants.CONTENT_BLOCK_SIZE+Constants.HASH_SIZE];
	    fis.read(buffer,Constants.HASH_SIZE,Constants.CONTENT_BLOCK_SIZE);
	    fis.close();
	    System.arraycopy(message,
			     0,
			     buffer,
			     0,
			     Constants.HASH_SIZE);
	    new NotificationThread(buffer).start();
	} catch (Exception e) {
	    System.err.println("caught exception: " + e.toString());
	    System.exit(-1);
	}
    }
    
    /**
     * Close the connection to the socket.
     **/
    public void close(){
	// nothing to do
    }
    
    /**
     * Thread that notifies each Listener for an
     * incoming packet.
     **/
    class NotificationThread extends Thread {
	
	/**
	 * The incoming packet.
	 **/
	private final byte[] buffer_;
	
	/**
	 * Create the Thread.
	 * @param buffer the incoming packet
	 **/
	NotificationThread (byte[] buffer) {
	    this.buffer_ = buffer;
	}

	/**
	 * The main method of the Thread. Calls
	 * each Listener and then exits.
	 **/
	public void run() {
	    // System.err.println("GProxy: receiving data"+listCount_);
	    byte[] data = new byte[Constants.CONTENT_BLOCK_SIZE];	
	    byte[] hash = new byte[Constants.HASH_SIZE];
	    System.arraycopy(buffer_,0,
			     hash,0,
			     Constants.HASH_SIZE);
	    System.arraycopy(buffer_,Constants.HASH_SIZE,
			     data,0,
			     Constants.CONTENT_BLOCK_SIZE);
	    synchronized (listeners_) {
		for (int i=0;i<listCount_;i++) {
		    try {
			listeners_[i].receive(hash, data);
		    } catch (Throwable t) {
			if (GProxy.DEBUG) {
			    System.err.println("BuildThread received exception: " + t);
			    t.printStackTrace();
			}
		    }
		}
	    }
	}

    } // end of NotificationThread
    
} // end of BuildThread
