/**
 * @file gproxy/MainWindow.java
 **/
package gproxy;

/**
 * This is the class representing the Main window of the
 * GProxy GUI.
 * @author Christian Grothoff
 **/
public class MainWindow 
    extends javax.swing.JFrame {

    /**
     * Length (in chars) of the default search query input window
     * (more characters may be entered anyway)
     **/
    private static final int SEARCH_TEXT_LEN = 20;

    /**
     * The thread used to request data.
     **/
    private final BuildThread bt_;

    /**
     * The download manager reponsible to display
     * active downloads.
     **/
    private final DownloadModel dlm_;

    /**
     * SearchTextField in the main window.
     **/
    private final javax.swing.JTextField searchTextField_;

    /** 
     * Create a new MainWindow.
     **/
    public MainWindow(DownloadModel dlm,
		      BuildThread bt) {
	this.bt_ = bt;
	this.dlm_ = dlm;
	getContentPane().setLayout(new java.awt.FlowLayout());
        setTitle("GProxy");
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing
		(java.awt.event.WindowEvent evt) {
		synchronized(bt_) {
		    bt_.close();
		    System.exit(0);
		}
            }
        });  

        this.searchTextField_
	    = new javax.swing.JTextField(SEARCH_TEXT_LEN);        
        searchTextField_.setEditable(true);
        searchTextField_.addActionListener
	    (new java.awt.event.ActionListener() {
            public void actionPerformed
		(java.awt.event.ActionEvent evt) {
                searchComboBoxActionPerformed();
            }
        });        
        getContentPane().add(searchTextField_);

 	javax.swing.JButton searchButton 
	    = new javax.swing.JButton();
        searchButton.setToolTipText("click here to start searching");
        searchButton.setText("search");
	searchButton.addActionListener
	    (new javax.swing.AbstractAction() {
		public void actionPerformed
		    (java.awt.event.ActionEvent e) {
		    searchComboBoxActionPerformed();
		}
	    });
	getContentPane().add(searchButton);

  	javax.swing.JButton quitButton 
	    = new javax.swing.JButton();        
        quitButton.setToolTipText("click here to exit");
        quitButton.setText("quit");
	quitButton.addActionListener(new javax.swing.AbstractAction() {
		public void actionPerformed(java.awt.event.ActionEvent e) {
		    System.exit(0);
		}
	    });
        getContentPane().add(quitButton);
	pack();
    }

    /**
     * Start the search.
     **/
    private void searchComboBoxActionPerformed() {
	String query 
	    = searchTextField_.getText();
	QueryModel qm
	    = new QueryModel(dlm_);
	QueryParser qp 
	    = new QueryParser(query, qm);
	QueryCoordinator qc 
	    = new QueryCoordinator(qp,
				   bt_); // will auto-start download
 	QueryFrame qf 
	    = new QueryFrame(qm, query, qc);
    }


} // end of MainWindow
