/**
 * Configuration management.
 * @author Christian Grothoff
 * @file gnet/configuration/configuration.c
 **/

#ifndef CONFIGURATION_H
#define CONFIGURATION_H

#include "gnettypes.h"

#define DEFAULT_CONFIG_FILE "~/.gnet"

/**
 * Define the maximum allowed network interfaces 
 **/
#define MAX_INTERFACES 20

/**
 * Maximum number of concurrent gproxies accessing this node.
 * Length of the sockDescriptors array.
 **/
#define CONCOUNT 4

/**
 * This method must be called first! It reads
 * the config file and makes everything else
 * possible.
 **/
void readConfig(FileName filename);

/**
 * This method may be called at last to clean up.
 * Afterwards everything but readConfig will result
 * in errors...
 **/
void freeConfig();


/* the rest should be clear... */

unsigned int getDiskQuota();

unsigned int getMaxLoad();

unsigned int getMaxNetLoad();

unsigned int getMaxNodes();

unsigned int getMaxNet();

unsigned int getMaxMem();

/**
 * How many hours is a HELO message
 * valid?
 **/
unsigned int getHeloExpiration();

unsigned short getGnetPort();

FileName getHostKeyFilePriv();

FileName getDatabaseFile();

FileName getCollisionFile();

FileName getSharedFileList();

DirName * getCreditDirectory();

DirName * getContentDirectory();

DirName * getNetworkIdDirectory();

IPAddress getIP();

Interfaces getNetInterfaces();
 
#endif
/* end of configuration.h */
