/**
 * The server side of the TCP connection between GProxy and Gnet node
 * @file gnet/node/test/TCPListener.c
 * @author Tiberiu Stef
*/


#include <stdio.h>
#include <stdlib.h> 				/* malloc */
#include <unistd.h>				/* for reading/writing data from/to socket descriptor */
#include <pthread.h>			/* thread stuff */
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h> 		/* for  sockaddr_in*/

#define INBUFFSIZE 512

/**
*	Handle data available on the TCP socket descriptor :
*	it's a simple echo, will return back everuthing received
*/
processData(int sockDescriptor){
	void* buff;
	int n;
	buff=malloc(INBUFFSIZE);
	if(buff==NULL){
		fprintf(stderr,"ERROR: %s Cannot allocate memory for incoming TCP buffer\n",__FILE__);
	}

	while (1){
		n= read(sockDescriptor,buff,INBUFFSIZE);
		if (n==0) return;
		if (n<0) {
			fprintf(stderr,"%s Error reading from socket\n",__FILE__);
		}
#ifdef DEBUG_TCP
		printf("Data from %d: %s\n",sockDescriptor, buff);
#endif
		/* send the data back - echo */
		write(sockDescriptor,buff,n);
		bzero(buff,INBUFFSIZE);
	}
}

/**
*	Method called whenever a new thread is created, when there is a new connection to the node
*	Will echo the data sent and then exit
*/
void incomingProcessor(int descriptor){

#ifdef DEBUG_TCP
	printf("Processing thread for descriptor %d\n",descriptor);
#endif
	processData(descriptor);
#ifdef DEBUG_TCP
	printf("Terminating thread %d...\n",descriptor);
#endif
	close(descriptor);
	pthread_exit(NULL);
}


/*************************
*		MAIN
************************/

main(int argc, char* argv[]){

	int listenerFD,incomingFD;
	int lenOfIncomingAddr;
	int listenerPort=6666;
    struct sockaddr_in serverAddr, clientAddr;

	int childPID; /* for the subprocess handling a new connection */

	pthread_t incomingThread; 	/* thread handling a new connection */
	int x;

    printf("TCP Listener starting ...\n");

	/* create the socket */
	if( (listenerFD=socket(PF_INET,SOCK_STREAM,0)) < 0){
		fprintf(stderr,"Error opening TCP listener\n");
        /* TODO: do I really wanna shutdown the program ? */
		exit(0);
    }
#ifdef DEBUG_TCP
    printf("DEBUG: %s: TCP Listener descriptor: %d\n",__FILE__,listenerFD);
#endif

	/* fill in the inet address structure */

	bzero((char *) &serverAddr, sizeof(serverAddr));
	serverAddr.sin_family = AF_INET;
	serverAddr.sin_addr.s_addr=htonl(INADDR_ANY);
	serverAddr.sin_port=htons(listenerPort);

    /*bind the socket */
	if(bind (listenerFD, (struct sockaddr *) &serverAddr,sizeof(serverAddr)) < 0){
		fprintf(stderr,"Error binding the TCP listener to port %d\n",listenerPort);
        /* TODO: do I really wanna shutdown the program ? */
		exit(0);
	}

	/* start listening for new connections */
	listen(listenerFD,5);

	/*process incoming data */
	/* For now just echo them back */
	while (1) {
		/*wait for a connection and process it */
		lenOfIncomingAddr=sizeof(clientAddr);
		incomingFD=accept(listenerFD,(struct sockaddr *)&clientAddr, &lenOfIncomingAddr);
		if(incomingFD <0){
			fprintf(stderr,"Error accepting new connection\n");
		}
		/* case 1: dedicated server
		processData(incomingFD);
		*/
		/* case 2: parallel server, forking subprocesses
		if( (childPID=fork()) < 0){
			fprintf(stderr,"Error creating subprocess to handle new incoming connection\n");
			// FIXME: really do this ?
			close(incomingFD);
		} else { 			//handle the subprocesses
			if (childPID==0) { 			// the child
				close(listenerFD);
				processData(incomingFD);
				close(incomingFD);
				exit(0);
			}
												//  the parent
			close (incomingFD);
		}
		*/

		/* case 3: parallel server, creating threads */


		if ((pthread_create(&incomingThread, NULL,
									(void * (*)(void *)) incomingProcessor, (void *) incomingFD)) != 0){
			fprintf(stderr,"Error creating thread to handle new incoming connection\n");
			// FIXME: really do this ?
			close(incomingFD);
		}
		// the whole control has been passed to incomingProcessor method.
	}

	printf("I shoud not see this ....Exiting...\n");
    exit(0);
}
