/* helotest.c - will test the helo protocol*/

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <fcntl.h>
#include <signal.h>
#include <dirent.h>
#include <string.h>
#include <errno.h>

#define TEST_DEBUG_F

#define TEST_DEBUG

#define GNETCONF_1 "./gnet.conf.1"
#define GNETCONF_2 "./gnet.conf.2"
#define GNETSITE_1_DATADIR ".gnet_1/data"
#define GNETSITE_2_DATADIR ".gnet_2/data"
#define GNETSITE_1_DIR ".gnet_1"
#define GNETSITE_2_DIR ".gnet_2"
#define GNETSITE_1_HOSTDIR ".gnet_1/data/hosts"
#define GNETSITE_2_HOSTDIR ".gnet_2/data/hosts"
#define GNETCODEPATH	"../gnet"

#define RM "/bin/rm"
#define CPY "/bin/cp"
#define MAKEDIR "/bin/mkdir"




int setupHosts(){
	struct stat confFileStat;
	DIR* hostOneDataDir;
	DIR* hostTwoDataDir;
	struct dirent* hostOneDataDirEntry;
	struct dirent* hostTwoDataDirEntry;
	int childpid1,childpid2;
	char srcFileName[256]="";
	char destFileName[256]="";
	char destFileName2[256]="";
	char tmpFileName[100]="";
	char fileBuff[1024];
	int fd1,fd2;
	int aCounter;

	printf("Setting up host\n");
	/* check existing conf file */
	if ((stat(GNETCONF_1,&confFileStat))!=0){
#ifdef TEST_DEBUG
		printf("ERROR: Config file for site 1 does not exist\n");
#endif
		return -1;
	}
	if ((stat(GNETCONF_2,&confFileStat))!=0){
#ifdef TEST_DEBUG
		printf("ERROR: Config file for site 1 does not exist\n");
#endif
		return -1;
	}
	/* fork and load each site */
	switch( childpid1=fork() ){
	case -1:
#ifdef TEST_DEBUG
		printf("ERROR: Could not fork a child\n");
#endif
		return -1;
	case 0:
		/* this is the child */
#ifdef TEST_DEBUG
		printf(" <-  Starting first node...\n");
#endif
		execlp(GNETCODEPATH,"gnet",GNETCONF_1,(char *)0);
		return 0;
	default:
		/* this is the parent, will fork a new child */
		aCounter=0;

		while((stat(GNETSITE_1_HOSTDIR,&confFileStat))==-1){
			if (aCounter==10){
#ifdef TEST_DEBUG
				printf(" I waited too much on hostkey generation. CYA\n");
#endif
				return -1;
			}
			aCounter ++;
			sleep(3);
		}
#ifdef TEST_DEBUG
		if(aCounter<10)
			printf("Node 1 finished building his key\n");
#endif
		sleep(60);
		/* Parent will put into second node's host dir, first node's key */
		if((hostOneDataDir=opendir(GNETSITE_1_HOSTDIR))==NULL){
#ifdef TEST_DEBUG
			printf("Cannot open Node 1 data directory, abort\n");
#endif
			return -1;
		}
		while((hostOneDataDirEntry=readdir(hostOneDataDir))!=NULL){
			if(hostOneDataDirEntry->d_name[0]=='.') continue;
			/* if(strncmp(hostOneDataDirEntry->d_name,"content",7)==0) continue; */
#ifdef TEST_DEBUG
			printf("Process entry %s\n",hostOneDataDirEntry->d_name);
#endif
			/* there should be only one key file here */
			break;
		}
		if((closedir(hostOneDataDir))==-1){
#ifdef TEST_DEBUG
			printf("Error closing site 2 host dir\n");
#endif
			return -1;
		}

		strcat(tmpFileName,hostOneDataDirEntry->d_name);
		strcat(srcFileName,GNETSITE_1_HOSTDIR);
		strcat(srcFileName,"/");
		strcat(srcFileName,hostOneDataDirEntry->d_name);
#ifdef TEST_DEBUG
		printf("srcFileName: %s\n",srcFileName);
#endif
		if((fd1=open(srcFileName,O_RDONLY))==-1){
#ifdef TEST_DEBUG
			printf("Error opening source signature file\n");
#endif
			return -1;
		}
#ifdef TEST_DEBUG
		printf("Source file opened\n");
#endif
		makeNodeTwoHostDir();
		strcat(destFileName,GNETSITE_2_HOSTDIR);
		strcat(destFileName,"/");
		strcat(destFileName,hostOneDataDirEntry->d_name);
#ifdef TEST_DEBUG
		printf("destFileName: %s \n",destFileName);
#endif
		if((stat(destFileName,&confFileStat))==0){
#ifdef TEST_DEBUG
			printf("PARENT: Signature already there, continue\n");
#endif
		}else{

			if((fd2=open(destFileName,O_RDWR | O_CREAT, S_IRWXU ))==-1){
#ifdef TEST_DEBUG
				printf("Error opening destination signature file\n");
#endif
				return -1;
			}

			if ((stat(srcFileName,&confFileStat))!=0){
#ifdef TEST_DEBUG
				printf("ERROR: Config file for site 1 does not exist\n");
#endif
				return -1;
			}

			aCounter=read(fd1,&fileBuff,confFileStat.st_size);
#ifdef TEST_DEBUG
			printf("FILE SIZE: %d, read from it: %d\n",confFileStat.st_size,aCounter);
#endif
			aCounter=write(fd2,&fileBuff,confFileStat.st_size);
#ifdef TEST_DEBUG
			printf("Wrote to file: %d\n",aCounter);
#endif

			if((close(fd2))==-1){
#ifdef TEST_DEBUG
				printf("ERROR Closing Destination file descriptor\n");
#endif
				return -1;
			}


			if((close(fd1))==-1){
#ifdef TEST_DEBUG
				printf("ERROR Closing Source file descriptor\n");
#endif
				return -1;
			}

#ifdef TEST_DEBUG
			printf("Done putting node 1's key into node 2's directory\n");
#endif
		} /* now I have the signature where is needed */

		printf("START SECOND NODE\n");
		sleep(5);
		/* start up the second node */
		switch(childpid2=fork()){
		case -1:
#ifdef TEST_DEBUG
			printf("ERROR: Could not fork a second child\n");
#endif
			return -1;
		case 0:
			// this is the second child
#ifdef TEST_DEBUG
			printf("Starting second node ..\n");
#endif
			execlp(GNETCODEPATH,"gnet",GNETCONF_2,(char *)0);
			return 0;

		default:
			// this is still the parent
#ifdef TEST_DEBUG
			printf("PARENT: SLEEP- let nodes settle %d, %d\n",childpid1,childpid2);
#endif
			sleep(120);
#ifdef TEST_DEBUG
			printf("PARENT WOKE UP..\n");
			printf("Kill the nodes and clean up...\n");
#endif
			// terminate the child processes and clean up
			kill(childpid1,SIGKILL);
			kill(childpid2,SIGKILL);

			/* now see if it has transferred the host key file */

			if((hostTwoDataDir=opendir(GNETSITE_2_HOSTDIR))==NULL){
#ifdef TEST_DEBUG_X
				printf("Cannot open Node 2 data directory, abort\n");
#endif
			return -1;
			}

			printf("Comparing against: %s\n",tmpFileName);

			while((hostTwoDataDirEntry=readdir(hostTwoDataDir))!=NULL){
				if(hostTwoDataDirEntry->d_name[0]=='.') continue;

				if(strncmp(hostTwoDataDirEntry->d_name,tmpFileName,10)==0) {
#ifdef TEST_DEBUG
					printf("Skipping keyfile that got moved: %s\n",tmpFileName);
#endif
					continue;
				}else{
#ifdef TEST_DEBUG
					printf("New key: %s\n",hostTwoDataDirEntry->d_name);
#endif
					break;
				}
			}
			if((closedir(hostTwoDataDir))==-1){
#ifdef TEST_DEBUG_X
				printf("Error closing site 2 host dir\n");
#endif
				return -1;
			}
#ifdef TEST_DEBUG
			printf("Key found: %s\n",hostTwoDataDirEntry->d_name);
#endif
			strcat(destFileName2,GNETSITE_1_HOSTDIR);
			strcat(destFileName2,"/");
			strcat(destFileName2,hostTwoDataDirEntry->d_name);
#ifdef TEST_DEBUG
			printf("Will look for %s in GNET1 host dir\n",destFileName2);
#endif
			if((stat(destFileName2,&confFileStat))!=0){
#ifdef TEST_DEBUG_F
				printf("ERROR: HELO WORKS  NOT  :( \n");
#endif
				/* clean up*/
				/* execlp(RM,RM,"-rf",".gnet_*",(char*)0); */
				return -1;
			}else{
#ifdef TEST_DEBUG_F
				printf("Parent process finished CORRECTLY, HELO WORKS \n");
#endif
				/* clean up*/
				/* execlp(RM,RM,"-rf",".gnet_*",(char*)0); */
				return 0;
			}
  		}
#ifdef TEST_DEBUG
		printf("Parent process finished prematurely\n");
#endif
		kill(childpid1,SIGKILL);
		execlp(RM,RM,"-rf",GNETSITE_1_DIR,(char*)0);
		return 0;
	} //from first fork
	kill(childpid1,SIGKILL);
	kill(childpid2,SIGKILL);
	return 0;
}






int makeNodeTwoHostDir(){

	struct stat confFileStat;
	const char* nodeTwoDir1=".gnet_2";
	const char* nodeTwoDir2=".gnet_2/data";
	const char* nodeTwoDir3=".gnet_2/data/hosts";
	int errorNo;

	if ((stat(nodeTwoDir1,&confFileStat))!=0){
#ifdef TEST_DEBUG_DIR
		printf("site 2 dir did not exist\n");
#endif
		errorNo=mkdir(nodeTwoDir1,S_IXUSR | S_IWUSR | S_IRUSR );
#ifdef TEST_DEBUG_DIR
		if(errorNo==-1){
			printf("ERROR: %s\n",strerror(errno));
		}
#endif
	}
	if ((stat(nodeTwoDir2,&confFileStat))!=0){
#ifdef TEST_DEBUG_DIR
		printf("site 2 DATA dir did not exist\n");
#endif
		errorNo=mkdir(nodeTwoDir2,S_IXUSR | S_IWUSR | S_IRUSR );
#ifdef TEST_DEBUG_DIR
		if(errorNo==-1){
			printf("ERROR: %s\n",strerror(errno));
		}
#endif
	}
		if ((stat(nodeTwoDir3,&confFileStat))!=0){
#ifdef TEST_DEBUG_DIR
		printf("site 2 hosts dir did not exist\n");
#endif
		errorNo=mkdir(nodeTwoDir3,S_IXUSR | S_IWUSR | S_IRUSR );
#ifdef TEST_DEBUG_DIR
		if(errorNo==-1){
			printf("ERROR: %s\n",strerror(errno));
		}
#endif
	}
}


main(){

  if (setupHosts() == 0)
    return 0;
  else{
    printf("\nTEST FAILED!\n\n");
    return -1;
  }
} // end of main
