/**
 * This module is responsible for breaking GNet. Er, for
 * handling collisions.
 * @file gnet/primitives/lookup_collision.c
 * @author Krista Bennett
 **/
 
#include "lookup_collision.h"

/* handle of the database-file, never closed yet */
int LOOKUP_COLLISION_handle_;

/* semaphore for synced r/w access to the file */
Semaphore * LOOKUP_COLLISION_fileSem_;

void initLookup_collision() {
  FileName fileName = getCollisionFile();
  fileName = expandFileName(fileName);
  LOOKUP_COLLISION_handle_ = open(fileName, O_CREAT|O_RDWR,S_IRUSR|S_IWUSR);
  if (LOOKUP_COLLISION_handle_ < 0) 
    errexit("Unable to open collision file %s. Exiting.\n", fileName);
  free(fileName);
  LOOKUP_COLLISION_fileSem_ = new_semaphore(1);
}


int findCollisionEntry(HashCode160 * hc,
		       ContentIndex * entry) {
   int size;
   HashCode160 hash_again;
   semaphore_down(LOOKUP_COLLISION_fileSem_);
   lseek(LOOKUP_COLLISION_handle_, 0, SEEK_SET);
   for (size = read(LOOKUP_COLLISION_handle_, entry, sizeof(ContentIndex)); 
        size == sizeof(ContentIndex); 
        size = read(LOOKUP_COLLISION_handle_, entry, sizeof(ContentIndex))) {
     hash(&entry->doubleHash,
	  sizeof(HashCode160),
	  &hash_again);       
     if (equalsHashCode160(&hash_again, hc)) {
       semaphore_up(LOOKUP_COLLISION_fileSem_);
       return 1;
     }     
   } 
   semaphore_up(LOOKUP_COLLISION_fileSem_);
   return 0;
}

void addCollisionEntry(ContentIndex * entry) {
   ContentIndex existing_entry;
   int position;
   int size;

   position = 0;
   semaphore_down(LOOKUP_COLLISION_fileSem_);
   lseek(LOOKUP_COLLISION_handle_, 0, SEEK_SET);
 
   for (size = read(LOOKUP_COLLISION_handle_, &existing_entry, sizeof(ContentIndex));
	size == sizeof(ContentIndex);
	size = read(LOOKUP_COLLISION_handle_, &existing_entry, sizeof(ContentIndex))) { 
     if (equalsHashCode160(&existing_entry.doubleHash,&entry->doubleHash)) 
       break;      
     if (size == sizeof(ContentIndex)) 
       position += size;
   }
   /* if we broke out of loop, this overwrites. Otherwise, we append to */
   /* end of file */
   lseek(LOOKUP_COLLISION_handle_, position, SEEK_SET); 
   write(LOOKUP_COLLISION_handle_, entry, sizeof(ContentIndex));
   semaphore_up(LOOKUP_COLLISION_fileSem_);
}
